class Client {
  private client : WebSocket;
  private connected : boolean = false;
  private progress : HTMLProgressElement;

  constructor () {
    let uri : string = 'ws://localhost:8082';
    this.progress = document.getElementById('progress') as HTMLProgressElement;
    this.client = new WebSocket(uri);
    this.client.onopen = this.onOpen.bind(this);
    this.client.onmessage = this.onMessage.bind(this);
  }

  private onMessage (event : any) {
    const msg : Message = JSON.parse(event.data) as Message;
    if (typeof msg.state !== 'undefined' && msg.state.sequence !== null) {
      this.setSequence(msg.state.sequence);
    } else if (typeof msg.cmd !== 'undefined' && msg.cmd !== null) {
      this.cmd(msg.cmd);
    }
  }

  private onOpen (event : any) {
    console.log('Connected');
    this.connected = true;
  }

  private setSequence(sequence : SequenceState) {
    const percent : number = sequence.progress * 100.0;
    this.progress.value = percent;
    this.progress.innerText = `${Math.floor(percent)}%`;
  }

  private cmd (id : string) {
    switch (id) {
      case 'open' :
        this.receiveCameraOpen();
        break;
      case 'close' :
        this.receiveCameraClose();
        break;
      default:
        console.warn(`No command ${id}`);
        break;
    }
  }

  public sendCameraOpen () {
    console.log('send camera open');
    this.client.send(JSON.stringify({ cmd : 'open' }));
  }

  private receiveCameraOpen () {
    console.log('got camera open');
  }

  public sendCameraClose () {
    console.log('send camera close');
    this.client.send(JSON.stringify({ cmd : 'close' }));
  }

  private receiveCameraClose () {
    console.log('got camera close');
  }

  public sendSelect () {
    const sequence : string = (document.getElementById('sequence') as HTMLSelectElement ).value;
    let msg : Message;
    if (sequence === '- Select Image Sequence -') {
      return;
    }
    msg = { cmd : 'select', sequence };
    console.log('send select');
    console.log(sequence)
    this.client.send(JSON.stringify(msg));
  }

  public fullscreen () {
    if (!document.fullscreenElement) {
      document.documentElement.requestFullscreen();
    } else {
      this.exitFullscreen();
    }
  }

  public exitFullscreen () {
    if (document.fullscreenElement) {
      document.exitFullscreen()
    }
  }


}

const client : Client = new Client();